package com.halomobi.sdk;

import static com.halomobi.common.ConstantPool.TrackerType.CLICK;
import static com.halomobi.common.ConstantPool.TrackerType.DEEP_LINK_OPEN;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ActivityNotFoundException;
import android.content.ComponentName;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.Bitmap;
import android.net.Uri;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebResourceRequest;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ImageView;

import com.halomobi.common.ConstantPool;
import com.halomobi.network.listeners.AdStateChangListener;
import com.halomobi.network.listeners.ClickAdStateChangListener;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.network.request.ad.bean.AdParameter;
import com.halomobi.sdk.listener.BannerListener;
import com.halomobi.sdk.listener.InsertListener;
import com.halomobi.utils.LogUtils;
import com.halomobi.utils.PxUtil;
import com.halomobi.utils.WebViewUtils;

import java.net.URISyntaxException;
import java.util.List;

public class HMBannerAd {
    private HlAdParameter mHlParameter;
    private AdController adController;
    private AdParameter mAdParameter;
    private Activity mActivity;
    private BannerListener mListener;
    private boolean isShow;
    private boolean isReady;
    private double currentPrice = 0;
    private Ad mAdInfo;

    public HMBannerAd(Activity activity, HlAdParameter parameter, BannerListener listener) {
        mActivity = activity;
        isReady = true;
        isShow = true;
        mListener = listener;
        mHlParameter = parameter;
        adController = new AdController(activity);
        mAdParameter = new AdParameter(HaloAdSdk.mAppId, parameter, activity.getPackageName(), ConstantPool.AdType.BANNER);
    }

    public void loadAd() {
        adController.requestAd(mAdParameter, new AdStateChangListener() {
            @Override
            public void onStart() {

            }

            @Override
            public void onAdError(String errorMsg, int errorCode) {
                mListener.onAdError(errorMsg, errorCode);
            }

            @Override
            public void onAdReach(Ad adInfo) {
                currentPrice = adInfo.price;
                mAdInfo = adInfo;
                if (isReady) {
                    isReady = false;
                    mListener.onAdReady();
                }
            }

            @Override
            public void onNativeAdReach(List<Ad> adInfoList) {

            }
        });
    }

    public void showView(ViewGroup viewGroup) {
        showBannerAd(mActivity, mAdInfo, viewGroup);
    }

    private void showBannerAd(Activity activity, Ad adInfo, ViewGroup viewGroup) {
        LogUtils.e("===adInfo===" + adInfo.adm);
        activity.runOnUiThread(() -> {
            String adid = adInfo.adid;
            String adm = adInfo.adm.replace("${AUCTION_BID_ID}", adid);
            View inflate = LayoutInflater.from(activity).inflate(R.layout.hl_dialog_banner, null, false);
            final WebView mWebView = inflate.findViewById(R.id.webView);
            ViewGroup.LayoutParams layoutParams = mWebView.getLayoutParams();
            if (mHlParameter.getAdWidth() > 0 && mHlParameter.getAdHeight() > 0) {
                layoutParams.width = mHlParameter.getAdWidth();
                layoutParams.height = mHlParameter.getAdHeight();
                mWebView.setLayoutParams(layoutParams);
            }
            WebViewUtils utils = new WebViewUtils();
            utils.initWebView(activity, mWebView, adInfo);
            mWebView.loadData(adm, "text/html", "UTF-8");
            viewGroup.addView(inflate);
            mWebView.setWebViewClient(new WebViewClient() {
                @Override
                public boolean shouldOverrideUrlLoading(WebView view, WebResourceRequest request) {
                    Log.e("11111", "===shouldOverrideUrlLoading===" + request.getUrl());
                    String url = request.getUrl().toString();
                    boolean isGoToApp = gotoQuickApp(url);
                    if(!isGoToApp) {
                        Intent intent = new Intent(mActivity, WebViewActivity.class);
                        intent.putExtra("urlStr", url);
                        mActivity.startActivity(intent);
                    }
                    return true;
                }

                @Override
                public void onPageStarted(WebView view, String url, Bitmap favicon) {
                    super.onPageStarted(view, url, favicon);

                }

                @Override
                public void onPageFinished(WebView view, String url) {
                    super.onPageFinished(view, url);
                    Log.e("11111", "===onPageFinished===" + url);
                    if (isShow) {
                        isShow = false;
                        mListener.onDisplayAd();
                    }
                }
            });
        });
    }

    private boolean gotoQuickApp(String deepLink) {
        try {
            Intent deepLinkIntent = Intent.parseUri(deepLink, Intent.URI_INTENT_SCHEME);
            deepLinkIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            ResolveInfo resolveInfo = mActivity.getPackageManager().resolveActivity(deepLinkIntent, PackageManager.MATCH_DEFAULT_ONLY);
            if (resolveInfo != null) {
                mActivity.startActivity(deepLinkIntent);
                return true;
            }
        } catch (ActivityNotFoundException e) {
            LogUtils.w("DeepLink NotFound  :  " + deepLink);
        } catch (URISyntaxException e) {
            e.printStackTrace();
            LogUtils.w("DeepLink NotFound  :  " + deepLink);
        }
        return false;
    }

    public double getEcpm() {
        return currentPrice;
    }
}
