package com.halomobi.sdk;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.ActivityNotFoundException;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.Bitmap;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.webkit.WebResourceRequest;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ImageView;

import com.halomobi.common.ConstantPool;
import com.halomobi.sdk.listener.InsertListener;
import com.halomobi.network.listeners.AdStateChangListener;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.network.request.ad.bean.AdParameter;
import com.halomobi.utils.LogUtils;
import com.halomobi.utils.PxUtil;
import com.halomobi.utils.WebViewUtils;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

public class HMInsertAd{
    private HlAdParameter mHlParameter;
    private AdController adController;
    private AdParameter mAdParameter;
    private Activity mActivity;
    private InsertListener mListener;
    private boolean isShow;
    private boolean isReady;
    private AlertDialog adDialog;
    private double currentPrice = 0;
    private Ad currentAd;

    public HMInsertAd(Activity activity, HlAdParameter parameter, InsertListener listener) {
        mActivity = activity;
        isReady = true;
        isShow = true;
        mListener = listener;
        mHlParameter = parameter;
        adDialog = new AlertDialog.Builder(activity).create();
        adDialog.setCancelable(false);
        adDialog.setOnDismissListener(DialogInterface::dismiss);
        adController = new AdController(activity);
        mAdParameter = new AdParameter(HaloAdSdk.mAppId, parameter, activity.getPackageName(), ConstantPool.AdType.INSERT);
    }

    public void loadAd() {
        adController.requestAd(mAdParameter, new AdStateChangListener() {
            @Override
            public void onStart() {

            }

            @Override
            public void onAdError(String errorMsg, int errorCode) {
                mListener.onAdError(errorMsg, errorCode);
            }

            @Override
            public void onAdReach(Ad adInfo) {
                currentAd = adInfo;
                currentPrice = adInfo.price;
                mListener.onAdReady();
            }

            @Override
            public void onNativeAdReach(List<Ad> adInfoList) {

            }
        });
    }

    public void showAd() {
        showInsertAd(mActivity, currentAd, mHlParameter);
    }

    private void showInsertAd(Activity activity, Ad adInfo, HlAdParameter hlParameter) {
        LogUtils.e("===adInfo===" + adInfo.adm + "====w===" + adInfo.w);
        activity.runOnUiThread(() -> {
            String adid = adInfo.adid;
            String adm = adInfo.adm.replace("${AUCTION_BID_ID}",adid);
            adDialog.show();
            Window window = adDialog.getWindow();
            if (adInfo.w > 10 && adInfo.h > 10) {
                if (window != null) {
                    WindowManager.LayoutParams lp = window.getAttributes();
                    lp.width = adInfo.w;
                    lp.height = adInfo.h;
                    lp.gravity = Gravity.CENTER;
                    adDialog.getWindow().setAttributes(lp);
                }
            } else {
                if (window != null) {
                    WindowManager.LayoutParams lp = window.getAttributes();
                    lp.width = hlParameter.getAdWidth();
                    lp.height = hlParameter.getAdHeight();
                    lp.gravity = Gravity.CENTER;
                    adDialog.getWindow().setAttributes(lp);
                }
            }
            View inflate = LayoutInflater.from(activity).inflate(R.layout.hl_dialog_insert, null, false);
            ImageView mImageClose = inflate.findViewById(R.id.image_close);
            final WebView mWebView = inflate.findViewById(R.id.webView);
            WebViewUtils utils = new WebViewUtils();
            utils.initWebView(activity, mWebView, adInfo);
            adDialog.setContentView(inflate);
            adDialog.setCancelable(true);
            mWebView.loadData(adm, "text/html", "UTF-8");
//            mWebView.loadUrl("https://www.baidu.com");
            mImageClose.setOnClickListener(v -> {
                adDialog.dismiss();
                mListener.onCloseAd();
            });

            mWebView.setWebViewClient(new WebViewClient() {
                @Override
                public boolean shouldOverrideUrlLoading(WebView view, WebResourceRequest request) {
                    Log.e("11111","===shouldOverrideUrlLoading===" + request.getUrl());
                    String url = request.getUrl().toString();
                    boolean isGoToApp = gotoQuickApp(url);
                    if(!isGoToApp) {
                        adDialog.dismiss();
                        Intent intent = new Intent(mActivity, WebViewActivity.class);
                        intent.putExtra("urlStr", url);
                        mActivity.startActivity(intent);
                    }
                    return false;
                }

                @Override
                public void onPageStarted(WebView view, String url, Bitmap favicon) {
                    super.onPageStarted(view, url, favicon);
                    if (isReady) {
                        isReady = false;
                        mListener.onAdReady();
                    }
                }

                @Override
                public void onPageFinished(WebView view, String url) {
                    super.onPageFinished(view, url);
                    mImageClose.setVisibility(View.VISIBLE);
                    Log.e("11111","===onPageFinished===" + url);
                    if (isShow) {
                        isShow = false;
                        mListener.onDisplayAd();
                    }
                }
            });
        });
    }

    private boolean gotoQuickApp(String deepLink) {
        try {
            Intent deepLinkIntent = Intent.parseUri(deepLink, Intent.URI_INTENT_SCHEME);
            deepLinkIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            ResolveInfo resolveInfo = mActivity.getPackageManager().resolveActivity(deepLinkIntent, PackageManager.MATCH_DEFAULT_ONLY);
            if (resolveInfo != null) {
                mActivity.startActivity(deepLinkIntent);
                adDialog.dismiss();
                return true;
            }
        } catch (ActivityNotFoundException e) {
            LogUtils.w("DeepLink NotFound  :  " + deepLink);
        } catch (URISyntaxException e) {
            e.printStackTrace();
            LogUtils.w("DeepLink NotFound  :  " + deepLink);
        }
        return false;
    }

    public double getEcpm() {
        return currentPrice;
    }
}
