package com.halomobi.sdk;

import android.app.Activity;
import android.content.Context;
import android.text.TextUtils;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.halomobi.common.ConstantPool;
import com.halomobi.sdk.listener.NativeAdListener;
import com.halomobi.sdk.listener.NativeResponse;
import com.halomobi.network.listeners.AdStateChangListener;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.network.request.ad.bean.AdParameter;
import com.halomobi.utils.LogUtils;
import com.halomobi.utils.Utils;
import com.halomobi.view.RadiusImageView;
import com.halomobi.view.imageloader.ImageLoader;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

public class HMNativeAd {
    private HlAdParameter mParameter;
    private int mDirectionType;
    private AdController adController;
    private NativeAdListener mNativeListener;
    private Activity mActivity;
    private NativeResponseImpl hlNativeResponse;

    public HMNativeAd(Activity activity, HlAdParameter parameter, NativeAdListener listener) {
        mActivity = activity;
        mParameter = parameter;
        mNativeListener = listener;
        mDirectionType = parameter.getType();
        adController = new AdController(activity);
    }

    public void loadAd() {
        AdParameter mAdParameter = new AdParameter(HaloAdSdk.mAppId, mParameter, mActivity.getPackageName(), ConstantPool.AdType.NATIVE);
        adController.requestAd(mAdParameter, new AdStateChangListener() {
            @Override
            public void onStart() {

            }

            @Override
            public void onAdError(String errorMsg, int errorCode) {
                mNativeListener.onAdError(errorMsg, errorCode);
            }

            @Override
            public void onAdReach(Ad adInfo) {
                LogUtils.e("===adInfo===" + adInfo.adm);
                analyseAd(adInfo);
                showAd();
            }

            @Override
            public void onNativeAdReach(List<Ad> adInfoList) {

            }
        });
    }

    private Ad analyseAd(Ad ad) {
        try {
            JSONObject jsonObject = new JSONObject(ad.adm);
            JSONObject aNative = jsonObject.optJSONObject("native");
            List<String> imgList = new ArrayList<>();
            hlNativeResponse = new NativeResponseImpl(adController);
            hlNativeResponse.setPrice(ad.price);
            if (aNative != null) {
                JSONArray assets = aNative.optJSONArray("assets");
                if (assets != null && assets.length() > 0) {
                    for (int i = 0; i < assets.length(); i++) {
                        JSONObject subObj = assets.getJSONObject(i);
                        JSONObject dataObject = subObj.optJSONObject("data");
                        if (dataObject != null) {
                            int type = dataObject.optInt("type");
                            String value = dataObject.optString("value");
                            if (type == 2) {
                                hlNativeResponse.setSubTitle(value);
                            }
                        }
                        JSONObject titleObject = subObj.optJSONObject("title");
                        if (titleObject != null) {
                            String value = titleObject.optString("text");
                            hlNativeResponse.setAdTitle(value);
                        }
                        JSONObject imgObject = subObj.optJSONObject("img");
                        if (imgObject != null) {
                            String imgUrl = imgObject.optString("url");
                            int type = imgObject.optInt("type");
                            if (type == 3) {
                                int w = imgObject.optInt("w");
                                if (w > 0) {
                                    hlNativeResponse.setImageWidth(w);
                                }
                                int h = imgObject.optInt("h");
                                if (h > 0) {
                                    hlNativeResponse.setImageHeight(h);
                                }
                                imgList.add(imgUrl);
                            }
                            if (type == 1) {
                                ad.lgsrc = imgUrl;
                            }
                        }
                    }
                }
                hlNativeResponse.setLogoUrl(ad.lgsrc);
                hlNativeResponse.setImgUrlList(imgList);
                ad.trackerGroup = new SparseArray<>();
                if (!TextUtils.isEmpty(ad.nurl)) {
                    String[] nurlArr = new String[]{ad.nurl};
                    ad.trackerGroup.put(ConstantPool.TrackerType.DISPLAY_NURL, nurlArr);
                }
                if (!TextUtils.isEmpty(ad.burl)) {
                    ad.trackerGroup.put(ConstantPool.TrackerType.DISPLAY_BURL, new String[]{ad.burl});
                }
                JSONArray imptrackers = aNative.optJSONArray("imptrackers");
                if (imptrackers != null && imptrackers.length() > 0) {
                    String[] imptrackerslist = addList(imptrackers);
                    ad.trackerGroup.put(ConstantPool.TrackerType.DISPLAY, imptrackerslist);
                }
                JSONObject linkObj = aNative.optJSONObject("link");
                if (linkObj != null) {
                    JSONArray clicktrackers = linkObj.optJSONArray("clicktrackers");
                    if (clicktrackers != null && clicktrackers.length() > 0) {
                        ad.trackerGroup.put(ConstantPool.TrackerType.CLICK, addList(clicktrackers));
                    }
                    String url = linkObj.optString("url");
                    ad.is_deep = true;
                    ad.dplk = url;
                    ad.clkurl = url;
                }
            }
            hlNativeResponse.setAdInfo(ad);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ad;
    }

    private void showAd() {
        HmNativeView hmNativeView = new HmNativeView(Utils.getContext());
        hmNativeView.showView(hlNativeResponse, mNativeListener);
        mNativeListener.onAdReach(hlNativeResponse, hmNativeView);
    }

    private String[] addList(JSONArray jsonArray) {
        String[] dataArray;
        try {
            int length = jsonArray.length();
            dataArray = new String[length];
            for (int i = 0; i < length; i++) {
                dataArray[i] = (String) jsonArray.opt(i);
            }
            return dataArray;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private class HmNativeView extends RelativeLayout {
        private View inflate;
        private boolean isShow;
        private boolean isClick;

        public HmNativeView(Context context) {
            super(context);
            isShow = false;
            if (mDirectionType == 1) {
                inflate = LayoutInflater.from(context).inflate(R.layout.native_item_imag_left, this);
            } else if (mDirectionType == 2) {
                inflate = LayoutInflater.from(context).inflate(R.layout.native_item_imag_right, this);
            } else if (mDirectionType == 3) {
                inflate = LayoutInflater.from(context).inflate(R.layout.native_item_imag_single, this);
            }
        }

        public void showView(NativeResponse hlResponse, final NativeAdListener mNativeListener) {
            RadiusImageView imageView = inflate.findViewById(R.id.iv_cion1);
            ImageView iv_close = inflate.findViewById(R.id.iv_close);
            TextView title = inflate.findViewById(R.id.tv_item_title);
            TextView subTitle = inflate.findViewById(R.id.tv_item_details);
            ImageView mImageSource = inflate.findViewById(R.id.image_item_source);
            if (hlResponse.getImageWidth() > 0 && hlResponse.getImageHeight() > 0) {
                ViewGroup.LayoutParams layoutParams = imageView.getLayoutParams();
                layoutParams.width = hlResponse.getImageWidth();
                layoutParams.height = hlResponse.getImageHeight();
                imageView.setLayoutParams(layoutParams);
            }
            String logoUrl = hlResponse.getLogoUrl();
            if (!TextUtils.isEmpty(logoUrl)) {
                mImageSource.setVisibility(View.VISIBLE);
                ImageLoader.load(mImageSource, logoUrl);
            } else {
                mImageSource.setVisibility(View.GONE);
            }
            List<String> imgList = hlResponse.getImgList();
            if (imgList != null && !imgList.isEmpty()) {
                ImageLoader.load(imageView, imgList.get(0));
            }
            if (!TextUtils.isEmpty(hlResponse.getTitle())) {
                title.setVisibility(View.VISIBLE);
                title.setText(hlResponse.getTitle());
            }
            if (!TextUtils.isEmpty(hlResponse.getAdDescription())) {
                subTitle.setVisibility(View.VISIBLE);
                subTitle.setText(hlResponse.getAdDescription());
            }
            inflate.setOnClickListener(v -> {
                hlResponse.handleClick(v, !isClick);
                if (!isClick) {
                    isClick = true;
                    if (mNativeListener != null) {
                        mNativeListener.onAdClick();
                    }
                }
            });

            iv_close.setOnClickListener(v -> mNativeListener.onAdClose());

            final RelativeLayout relativeLayout = inflate.findViewById(R.id.relativelayout);
            ViewTreeObserver viewTreeObserver = relativeLayout.getViewTreeObserver();
            viewTreeObserver.addOnGlobalLayoutListener(() -> {
                boolean viewCovered = Utils.isViewCovered(relativeLayout, 0.3f);
                if (!viewCovered && !isShow) {
                    isShow = true;
                    hlResponse.handleShow();
                    mNativeListener.onAdShow();
                }
            });

            viewTreeObserver.addOnScrollChangedListener(() -> {
                boolean viewCovered = Utils.isViewCovered(relativeLayout, 0.3f);
                if (!viewCovered && !isShow) {
                    isShow = true;
                    hlResponse.handleShow();
                    mNativeListener.onAdShow();
                }
            });
        }
    }
}