package com.halomobi.sdk;

import android.app.Activity;
import android.graphics.Color;
import android.text.TextUtils;
import android.util.Log;
import android.util.SparseArray;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.halomobi.common.ConstantPool;
import com.halomobi.sdk.listener.NativeAdListener;
import com.halomobi.sdk.listener.NativeUnifiedListener;
import com.halomobi.network.listeners.AdStateChangListener;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.network.request.ad.bean.AdParameter;
import com.halomobi.utils.PxUtil;
import com.halomobi.utils.Utils;
import com.halomobi.view.imageloader.ImageLoader;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

public class HMUnifiedNativeAd {
    private boolean isShow;
    private boolean isClick;
    private AdParameter mAdParameter;
    private AdController adController;
    private NativeAdListener mListener;
    private Activity mActivity;
    private NativeResponseImpl nativeResponse;

    public HMUnifiedNativeAd(Activity activity, HlAdParameter parameter, NativeAdListener listener) {
        mActivity = activity;
        mListener = listener;
        adController = new AdController(activity);
        mAdParameter = new AdParameter(HaloAdSdk.mAppId, parameter, activity.getPackageName(), ConstantPool.AdType.NATIVE);
    }

    public void loadAd() {
        adController.requestAd(mAdParameter, new AdStateChangListener() {
            @Override
            public void onStart() {

            }

            @Override
            public void onAdError(String errorMsg, int errorCode) {
                mListener.onAdError(errorMsg, errorCode);
            }

            @Override
            public void onAdReach(Ad adInfo) {
                nativeResponse = analyseAd(adInfo);
                mListener.onAdReach(analyseAd(adInfo), new View(mActivity));
            }

            @Override
            public void onNativeAdReach(List<Ad> adInfoList) {

            }
        });
    }

    private NativeResponseImpl analyseAd(Ad ad) {
        NativeResponseImpl hlNativeResponse = new NativeResponseImpl(adController);
        try {
            JSONObject jsonObject = new JSONObject(ad.adm);
            JSONObject aNative = jsonObject.optJSONObject("native");
            List<String> imgList = new ArrayList<>();
            if (aNative != null) {
                JSONArray assets = aNative.optJSONArray("assets");
                if (assets != null && assets.length() > 0) {
                    for (int i = 0; i < assets.length(); i++) {
                        JSONObject subObj = assets.getJSONObject(i);
                        JSONObject dataObject = subObj.optJSONObject("data");
                        if (dataObject != null) {
                            int type = dataObject.optInt("type");
                            String value = dataObject.optString("value");
                            if (type == 2) {
                                hlNativeResponse.setSubTitle(value);
                            }
                        }
                        JSONObject titleObject = subObj.optJSONObject("title");
                        if (titleObject != null) {
                            String value = titleObject.optString("text");
                            hlNativeResponse.setAdTitle(value);
                        }
                        JSONObject imgObject = subObj.optJSONObject("img");
                        if (imgObject != null) {
                            String imgUrl = imgObject.optString("url");
                            int type = imgObject.optInt("type");
                            if (type == 3) {
                                imgList.add(imgUrl);
                            }
                            if (type == 1) {
                                ad.lgsrc = imgUrl;
                            }
                        }
                    }
                }
                hlNativeResponse.setImgUrlList(imgList);
                ad.trackerGroup = new SparseArray<>();
                if (!TextUtils.isEmpty(ad.nurl)) {
                    String[] nurlArr = new String[]{ad.nurl};
                    ad.trackerGroup.put(ConstantPool.TrackerType.DISPLAY_NURL, nurlArr);
                }
                if (!TextUtils.isEmpty(ad.burl)) {
                    ad.trackerGroup.put(ConstantPool.TrackerType.DISPLAY_BURL, new String[]{ad.burl});
                }
                JSONArray imptrackers = aNative.optJSONArray("imptrackers");
                if (imptrackers != null && imptrackers.length() > 0) {
                    String[] imptrackerslist = addList(imptrackers);
                    ad.trackerGroup.put(ConstantPool.TrackerType.DISPLAY, imptrackerslist);
                }
                JSONObject linkObj = aNative.optJSONObject("link");
                if (linkObj != null) {
                    JSONArray clicktrackers = linkObj.optJSONArray("clicktrackers");
                    if (clicktrackers != null && clicktrackers.length() > 0) {
                        ad.trackerGroup.put(ConstantPool.TrackerType.CLICK, addList(clicktrackers));
                    }
                    String url = linkObj.optString("url");
                    ad.clkurl = url;
                    ad.landing_url = url;
                    ad.dplk = url;
                }
            }
            hlNativeResponse.setPrice(ad.price);
            hlNativeResponse.setAdInfo(ad);
        } catch (Exception e) {
            mListener.onAdError("解析失败:" + e, 100);
            e.printStackTrace();
        }
        return hlNativeResponse;
    }

    private String[] addList(JSONArray jsonArray) {
        String[] dataArray;
        try {
            int length = jsonArray.length();
            dataArray = new String[length];
            for (int i = 0; i < length; i++) {
                dataArray[i] = (String) jsonArray.opt(i);
            }
            return dataArray;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public View showViewById(int adLayoutId, int adContentId, int adTitleId, int adSubTitleId, int adOptionsId) {
        View inflate = View.inflate(mActivity, adLayoutId, null);
        RelativeLayout relativeLayout = new RelativeLayout(mActivity);
        ImageView imageView = new ImageView(mActivity);
        imageView.setScaleType(ImageView.ScaleType.FIT_XY);
        View mAdOptionsId = inflate.findViewById(adOptionsId);
        if (mAdOptionsId != null) {
            mAdOptionsId.setVisibility(View.GONE);
        }
        if (adContentId > 0) {
            ViewGroup mAdContentId = inflate.findViewById(adContentId);
            if (mAdContentId != null) {
                mAdContentId.post(() -> {
                    RelativeLayout.LayoutParams layoutParams = new RelativeLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
                    if (mAdContentId.getWidth() > 0) {
                        layoutParams.width = mAdContentId.getWidth();
                    } else  {
                        layoutParams.width = PxUtil.getDeviceWidthInPixel(mActivity);
                    }
                    layoutParams.height = mAdContentId.getHeight();
                    relativeLayout.setLayoutParams(layoutParams);
                    imageView.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));

                    mActivity.runOnUiThread(() -> {
                        List<String> imgList = nativeResponse.getImgList();
                        if (imgList != null && !imgList.isEmpty()) {
                            ImageLoader.load(imageView, imgList.get(0));
                        }
                        relativeLayout.addView(imageView);
                        mAdContentId.addView(relativeLayout);
                        if (adTitleId > 0) {
                            TextView mTitleAd = inflate.findViewById(adTitleId);
                            if (mTitleAd != null) {
                                mTitleAd.setText(nativeResponse.getTitle());
                            }
                        }
                        if (adSubTitleId > 0) {
                            TextView mSubTitle = inflate.findViewById(adSubTitleId);
                            if (mSubTitle != null) {
                                mSubTitle.setText(nativeResponse.getAdDescription());
                            }
                        }
                    });
                });
            }
        }

        inflate.setOnClickListener(v -> {
            nativeResponse.handleClick(v, !isClick);
            if (!isClick) {
                isClick = true;
                if (mListener != null) {
                    mListener.onAdClick();
                }
            }
        });

        ViewTreeObserver viewTreeObserver = inflate.getViewTreeObserver();
        viewTreeObserver.addOnGlobalLayoutListener(() -> {
            boolean viewCovered = Utils.isViewCovered(inflate, 0.3f);
            if (!viewCovered && !isShow) {
                isShow = true;
                nativeResponse.handleShow();
                mListener.onAdShow();
            }
        });

        viewTreeObserver.addOnScrollChangedListener(() -> {
            boolean viewCovered = Utils.isViewCovered(inflate, 0.3f);
            if (!viewCovered && !isShow) {
                isShow = true;
                nativeResponse.handleShow();
                mListener.onAdShow();
            }
        });
        return inflate;
    }
}
