package com.halomobi.sdk;

import static com.halomobi.network.request.ad.bean.Ad.SCREEN_ORIENTATION_LANDSCAPE;
import static com.halomobi.network.request.ad.bean.Ad.SCREEN_ORIENTATION_PORTRAIT;

import android.app.Activity;
import android.content.Intent;
import android.text.TextUtils;
import android.util.Log;
import android.util.SparseArray;

import com.halomobi.common.ConstantPool;
import com.halomobi.network.listeners.AdRedirectChangListener;
import com.halomobi.network.listeners.AdStateChangListener;
import com.halomobi.network.request.ad.RequestAdManager;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.network.request.ad.bean.AdParameter;
import com.halomobi.sdk.listener.RewardVideoListener;
import com.halomobi.utils.HlRewardVideoManager;
import com.halomobi.utils.LogUtils;
import com.halomobi.vast.XmlTools;
import com.halomobi.vast.model.TRACKING_EVENTS_TYPE;
import com.halomobi.vast.model.VASTMediaFile;
import com.halomobi.vast.model.VASTModel;
import com.halomobi.vast.model.VideoClicks;

import org.json.JSONArray;
import org.json.JSONObject;
import org.w3c.dom.Document;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.logging.Logger;

public class HMVideoAd {
    private RewardVideoListener mListener;
    private HlAdParameter mParameter;
    private Activity mActivity;
    private Ad mAdInfo;
    private double currentPrice;

    public HMVideoAd(Activity activity, HlAdParameter parameter, RewardVideoListener listener) {
        mActivity = activity;
        mParameter = parameter;
        mListener = listener;
    }

    public void loadAd() {
        AdController adController = new AdController(mActivity);
        AdParameter mAdParameter = new AdParameter(HaloAdSdk.mAppId, mParameter, mActivity.getPackageName(), ConstantPool.AdType.REWARDVIDEO);
        adController.requestAd(mAdParameter, new AdStateChangListener() {
            @Override
            public void onStart() {

            }

            @Override
            public void onAdError(String errorMsg, int errorCode) {
                mListener.onError(errorCode, errorMsg);
            }

            @Override
            public void onAdReach(Ad adInfo) {
                String adm = adInfo.adm;
                LogUtils.e("===adInfo===" + adInfo.adm);
                currentPrice = adInfo.price;
                adInfo.adm = adm.replace("${AUCTION_BID_ID}", adInfo.adid);
                adInfo.reward_video = new Ad.RewardVideoAds();
                adInfo.trackerGroup = new SparseArray<>();
                mAdInfo = analyseAd(adInfo);
            }

            @Override
            public void onNativeAdReach(List<Ad> adInfoList) {

            }
        });
    }

    public void showAd() {
        showAd(mActivity, mAdInfo, mListener);
    }

    private Ad analyseAd(Ad ad) {
        try {
            Document mainDoc = XmlTools.stringToDocument(ad.adm);
            VASTModel vastModel = new VASTModel(mainDoc);
            int duration = formatStringToLongData(vastModel.getDuration());
            ad.reward_video.videoDuration = String.valueOf(duration);
            List<VASTMediaFile> mediaFiles = vastModel.getMediaFiles();
            if (mediaFiles != null && !mediaFiles.isEmpty()) {
                VASTMediaFile vastMediaFile = mediaFiles.get(0);
                ad.reward_video.videoUrl = vastMediaFile.getValue();
                int width = vastMediaFile.getWidth();
                int height = vastMediaFile.getHeight();
                ad.reward_video.videoWidth = String.valueOf(width);
                ad.reward_video.videoHeight = String.valueOf(height);
                ad.reward_video.orientation = SCREEN_ORIENTATION_PORTRAIT;
                if (width > height) { //横屏
                    ad.reward_video.orientation = SCREEN_ORIENTATION_LANDSCAPE;
                }
                if (!vastModel.getAdTitle().isEmpty()) {
                    ad.reward_video.title = vastModel.getAdTitle().get(0);
                }
                if (!vastModel.getDescription().isEmpty()) {
                    ad.reward_video.subTitle = vastModel.getDescription().get(0);
                }
            }

            List<String> adParameters = vastModel.getAdParameters();
            if (adParameters != null && !adParameters.isEmpty()) {
                String s = adParameters.get(0);
                JSONObject object = new JSONObject(s);
                JSONArray mediaFilesArray = object.optJSONArray("mediaFiles");
                if (mediaFilesArray != null && mediaFilesArray.length() > 0) {
                    JSONObject jsonObject = mediaFilesArray.getJSONObject(0);
                    ad.reward_video.videoUrl = jsonObject.optString("uri");
                    int width = jsonObject.optInt("width");
                    int height = jsonObject.optInt("height");
                    ad.reward_video.orientation = SCREEN_ORIENTATION_PORTRAIT;
                    if (width > height) { //横屏
                        ad.reward_video.orientation = SCREEN_ORIENTATION_LANDSCAPE;
                    }
                    if (!vastModel.getAdTitle().isEmpty()) {
                        ad.reward_video.title = vastModel.getAdTitle().get(0);
                    }
                }
                int duration1 = object.optInt("duration");
                ad.reward_video.videoDuration = String.valueOf(duration1);
            }

            List<String> impressions = vastModel.getImpressions();
            String[] displayArray = ad.trackerGroup.get(ConstantPool.TrackerType.DISPLAY);
            ArrayList<String> displayList = new ArrayList<>();
            if (displayArray != null && displayArray.length > 0) {
                displayList.addAll(Arrays.asList(displayArray));
            }
            if (impressions != null && !impressions.isEmpty()) {
                displayList.addAll(impressions);
                ad.trackerGroup.put(ConstantPool.TrackerType.DISPLAY, displayList.toArray(new String[0]));
            }

            String[] clickArray = ad.trackerGroup.get(ConstantPool.TrackerType.CLICK);
            ArrayList<String> clickList = new ArrayList<>();
            if (clickArray != null && clickArray.length > 0) {
                clickList.addAll(Arrays.asList(clickArray));
            }
            VideoClicks videoClicks = vastModel.getVideoClicks();
            if (videoClicks != null) {
                List<String> clickTracking = videoClicks.getClickTracking();
                if (clickTracking != null && !clickTracking.isEmpty()) {
                    clickList.addAll(clickTracking);
                    ad.trackerGroup.put(ConstantPool.TrackerType.CLICK, clickList.toArray(new String[0]));//点击追踪
                }

                String clickThrough = videoClicks.getClickThrough();
                if (!TextUtils.isEmpty(clickThrough)) {
                    ad.is_deep = true;
                    ad.dplk = clickThrough;
                    ad.clkurl = clickThrough;
                }
            }


            HashMap<TRACKING_EVENTS_TYPE, List<String>> trackingUrls = vastModel.getTrackingUrls();
            if (trackingUrls != null && !trackingUrls.isEmpty()) {
                String[] closeArray = ad.trackerGroup.get(ConstantPool.TrackerType.CLOSE);
                ArrayList<String> closeList = new ArrayList<>();
                if (closeArray != null && closeArray.length > 0) {
                    closeList.addAll(Arrays.asList(closeArray));
                }
                List<String> close = trackingUrls.get(TRACKING_EVENTS_TYPE.close);
                if (close != null && !close.isEmpty()) {
                    closeList.addAll(close);
                    ad.trackerGroup.put(ConstantPool.TrackerType.CLOSE, closeList.toArray(new String[0]));//关闭追踪
                }

                String[] unmuteArray = ad.trackerGroup.get(ConstantPool.TrackerType.UNMUTE_TRACKER);
                ArrayList<String> unmuteList = new ArrayList<>();
                if (unmuteArray != null && unmuteArray.length > 0) {
                    unmuteList.addAll(Arrays.asList(unmuteArray));
                }
                List<String> unmute = trackingUrls.get(TRACKING_EVENTS_TYPE.unmute);
                if (unmute != null && !unmute.isEmpty()) {
                    unmuteList.addAll(unmute);
                    ad.trackerGroup.put(ConstantPool.TrackerType.UNMUTE_TRACKER, unmuteList.toArray(new String[0]));//静音
                }

                String[] muteArray = ad.trackerGroup.get(ConstantPool.TrackerType.MUTE_TRACKER);
                ArrayList<String> muteList = new ArrayList<>();
                if (muteArray != null && muteArray.length > 0) {
                    muteList.addAll(Arrays.asList(muteArray));
                }
                List<String> mute = trackingUrls.get(TRACKING_EVENTS_TYPE.mute);
                if (mute != null && !mute.isEmpty()) {
                    muteList.addAll(mute);
                    ad.trackerGroup.put(ConstantPool.TrackerType.MUTE_TRACKER, muteList.toArray(new String[0]));//取消静音
                }

//            List<String> pause = trackingUrls.get(TRACKING_EVENTS_TYPE.pause);
//            if (pause != null && !pause.isEmpty()) {
//                ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_PAUSE_TRACKER, pause.toArray(new String[0]));//播放暂停
//            }

                String[] skipArray = ad.trackerGroup.get(ConstantPool.TrackerType.VIDEO_SKIP_TRACKER);
                ArrayList<String> skipList = new ArrayList<>();
                if (skipArray != null && skipArray.length > 0) {
                    skipList.addAll(Arrays.asList(skipArray));
                }
                List<String> skip = trackingUrls.get(TRACKING_EVENTS_TYPE.skip);
                if (skip != null && !skip.isEmpty()) {
                    skipList.addAll(skip);
                    ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_SKIP_TRACKER, skipList.toArray(new String[0]));//视频跳过播放
                }

                String[] startArray = ad.trackerGroup.get(ConstantPool.TrackerType.VIDEO_START_TRACKER);
                ArrayList<String> startList = new ArrayList<>();
                if (startArray != null && startArray.length > 0) {
                    startList.addAll(Arrays.asList(startArray));
                }
                List<String> start = trackingUrls.get(TRACKING_EVENTS_TYPE.start);
                if (start != null && !start.isEmpty()) {
                    startList.addAll(start);
                    ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_START_TRACKER, startList.toArray(new String[0]));//播放开始
                }

                String[] firstArray = ad.trackerGroup.get(ConstantPool.TrackerType.VIDEO_PLAYING_1_TRACKER);
                ArrayList<String> firstList = new ArrayList<>();
                if (firstArray != null && firstArray.length > 0) {
                    firstList.addAll(Arrays.asList(firstArray));
                }
                List<String> firstQuartile = trackingUrls.get(TRACKING_EVENTS_TYPE.firstQuartile);
                if (firstQuartile != null && !firstQuartile.isEmpty()) {
                    firstList.addAll(firstQuartile);
                    ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_PLAYING_1_TRACKER, firstList.toArray(new String[0]));//视频播放1/4
                }

                String[] midpointArray = ad.trackerGroup.get(ConstantPool.TrackerType.VIDEO_PLAYING_2_TRACKER);
                ArrayList<String> midpointList = new ArrayList<>();
                if (midpointArray != null && midpointArray.length > 0) {
                    midpointList.addAll(Arrays.asList(midpointArray));
                }
                List<String> midpoint = trackingUrls.get(TRACKING_EVENTS_TYPE.midpoint);
                if (midpoint != null && !midpoint.isEmpty()) {
                    midpointList.addAll(midpoint);
                    ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_PLAYING_2_TRACKER, midpointList.toArray(new String[0]));//视频播放1/2
                }

                String[] thirdArray = ad.trackerGroup.get(ConstantPool.TrackerType.VIDEO_PLAYING_3_TRACKER);
                ArrayList<String> thirdList = new ArrayList<>();
                if (thirdArray != null && thirdArray.length > 0) {
                    thirdList.addAll(Arrays.asList(thirdArray));
                }
                List<String> thirdQuartile = trackingUrls.get(TRACKING_EVENTS_TYPE.thirdQuartile);
                if (thirdQuartile != null && !thirdQuartile.isEmpty()) {
                    thirdList.addAll(thirdQuartile);
                    ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_PLAYING_3_TRACKER, thirdList.toArray(new String[0]));//视频播放3/4
                }

                String[] completeArray = ad.trackerGroup.get(ConstantPool.TrackerType.VIDEO_COMPLETE_TRACKER);
                ArrayList<String> completeList = new ArrayList<>();
                if (completeArray != null && completeArray.length > 0) {
                    completeList.addAll(Arrays.asList(completeArray));
                }
                List<String> complete = trackingUrls.get(TRACKING_EVENTS_TYPE.complete);
                if (complete != null && !complete.isEmpty()) {
                    completeList.addAll(complete);
                    ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_COMPLETE_TRACKER, completeList.toArray(new String[0]));//播放完成
                }
            }
            List<String> adTagUri = vastModel.getAdTagUri();
            if (!adTagUri.isEmpty()) {
                ad.tagUriStr = adTagUri.get(0);
                Log.e("11111", "===tagUriStr===" + ad.tagUriStr);
                loadRedirectAd(ad.tagUriStr);
            } else if (!TextUtils.isEmpty(ad.reward_video.videoUrl)) {
                mListener.onSuccess();
            } else {
                mListener.onError(100, "未解析到视频地址");
            }

        } catch (Exception e) {
            mListener.onError(100, "xml解析失败：" + e);
            e.printStackTrace();
        }
        return ad;
    }

    private void showAd(Activity activity, Ad adInfo, RewardVideoListener listener) {
        HlRewardVideoManager instance = HlRewardVideoManager.getInstance();
        instance.mAd = adInfo;
        String videoUrl = adInfo.reward_video.videoUrl;
        LogUtils.e("====videoUrl===" + videoUrl);
        instance.isReady = true;
        instance.setListener(listener);
        Intent intent = new Intent(activity, RewardVideoActivity.class);
        activity.startActivity(intent);
    }

    private void loadRedirectAd(String tagUriStr) {
        RequestAdManager.getInstance().requestRedirectAd(tagUriStr, new AdRedirectChangListener() {
            @Override
            public void onAdError(String errorMsg, int errorCode) {
                Log.e("11111", "====error====");
                mListener.onError(errorCode, errorMsg);
            }

            @Override
            public void onXmlSuccess(String xmlStr) {
                mAdInfo.adm = xmlStr;
                analyseAd(mAdInfo);
                if (!TextUtils.isEmpty(mAdInfo.reward_video.videoUrl)) {
                    mListener.onSuccess();
                }
            }
        });
    }

    private int formatStringToLongData(String currentTime) {
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss", Locale.CHINESE);
            Date d = sdf.parse(currentTime);
            Calendar c = Calendar.getInstance();
            if (d != null) {
                c.setTime(d);
                int minute = c.get(Calendar.MINUTE);
                int second = c.get(Calendar.SECOND);
                return minute * 60 + second;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return 0;
    }

    public double getEcpm() {
        return currentPrice;
    }
}
