package com.halomobi.sdk;

import android.app.Activity;
import android.content.Context;
import android.os.CountDownTimer;
import android.text.TextUtils;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.halomobi.common.ConstantPool;
import com.halomobi.hlinterface.IAdControl;
import com.halomobi.sdk.listener.RewardVideoListener;
import com.halomobi.network.listeners.ClickAdStateChangListener;
import com.halomobi.network.report.track.ReportTrackerManager;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.utils.HlRewardVideoManager;
import com.halomobi.utils.LogUtils;
import com.halomobi.utils.Utils;
import com.halomobi.video.JZDataSource;
import com.halomobi.video.JZMediaManager;
import com.halomobi.video.JZUserAction;
import com.halomobi.video.Jzvd;
import com.halomobi.video.JZUtils;
import com.halomobi.view.HlTipsDialog;
import com.halomobi.view.imageloader.ImageLoader;

/**
 * @author Feng Zhanao
 * @date 2019/3/8
 * @description
 */
public class HlCustomView extends Jzvd {

    private final TextView mTvAd;
    private final ImageView mImageAd;
    ImageView mIvVolumn;
    TextView mTvClose,mTvTime;
    ProgressBar mProgressBar;
    private boolean isVolumn = true;
    private Activity mContext;
    private Ad mAd;
    private IAdControl mAdController;
    private RewardVideoListener mRewardVideoListener;
    private boolean isCallOnAdShow;
    View bannerView;
    private static VideoStateChangeListener mVideoStateChangeListener;
    private RelativeLayout relative_small, relative_big, relative_center;
    private boolean videoClose = false;
    private CountDownTimer timer = null;

    public HlCustomView(Context context) {
        super(context);
        mContext = (Activity) context;
        mAdController = new AdController(mContext);
        mAdController.setClickAdListener(mClickAdStateChangListener);
        mIvVolumn = findViewById(R.id.hl_iv_volume);
        mProgressBar = findViewById(R.id.hl_progress);
        mTvTime = findViewById(R.id.tv_time);
        mTvClose = findViewById(R.id.tv_close);
        mTvAd = findViewById(R.id.tv_ad);
        mImageAd = findViewById(R.id.image_ad);
        mIvVolumn.setOnClickListener(this);
        mTvClose.setOnClickListener(this);
        mRewardVideoListener = HlRewardVideoManager.getInstance().getListener();
    }

    @Override
    public int getLayoutId() {
        return R.layout.hl_custom_video;
    }

    @Override
    public boolean onTouch(View view, MotionEvent motionEvent) {
        if (motionEvent.getAction() == MotionEvent.ACTION_UP) {
            int[] screenSize = Utils.getScreenSize(true);
            float getY = motionEvent.getY();
            onclickDetail();
        }
        return false;
    }

    public static void startFullscreen(Context context, Class _class, String url, String title, String orientation, VideoStateChangeListener listener) {
        mVideoStateChangeListener = listener;
        startFullscreen(context, _class, new JZDataSource(url, title), orientation);
    }

    @Override
    public void onClick(View v) {
        int i = v.getId();
        if (i == R.id.hl_iv_volume) {
            if (isVolumn) {
                JZMediaManager.instance().jzMediaInterface.setVolume(0f, 0f);
                reportTracker(ConstantPool.TrackerType.MUTE_TRACKER);
                ((ImageView) v).setImageResource(JZUtils.getId("hj_mute", "drawable"));
                isVolumn = false;
            } else {
                JZMediaManager.instance().jzMediaInterface.setVolume(1f, 1f);
                reportTracker(ConstantPool.TrackerType.UNMUTE_TRACKER);
                ((ImageView) v).setImageResource(JZUtils.getId("hj_unmute", "drawable"));
                isVolumn = true;
            }
        } else if (i == R.id.hj_iv_start) {
            if (jzDataSource.urlsMap.isEmpty() || jzDataSource.getCurrentUrl() == null) {
                return;
            }
            if (currentState == CURRENT_STATE_NORMAL) {
                startVideo();
                onEvent(JZUserAction.ON_CLICK_START_ICON);
            } else if (currentState == CURRENT_STATE_PLAYING) {
                onEvent(JZUserAction.ON_CLICK_PAUSE);
                Log.d(TAG, "pauseVideo [" + this.hashCode() + "] ");
                JZMediaManager.pause();
            } else if (currentState == CURRENT_STATE_PAUSE) {
                onEvent(JZUserAction.ON_CLICK_RESUME);
                JZMediaManager.start();
                onStatePlaying();
            } else if (currentState == CURRENT_STATE_AUTO_COMPLETE) {
                onEvent(JZUserAction.ON_CLICK_START_AUTO_COMPLETE);
                startVideo();
            }
        } else if (i == R.id.tv_close) {
            if (videoClose) {
                mRewardVideoListener.onAdClose();
//                reportTracker(ConstantPool.TrackerType.CLOSE);
                mContext.finish();
            } else {
                showTipsDialog();
            }
        }
    }

    private void showTipsDialog() {
        HlTipsDialog tipsDialog = new HlTipsDialog(mContext, R.style.hl_dialog);
        tipsDialog.setonDialogClickListener(new HlTipsDialog.OnDialogClickListener() {
            @Override
            public void onCancelClick() {
                mRewardVideoListener.onAdSkip();
                reportTracker(ConstantPool.TrackerType.CLOSE);
                mContext.finish();
            }

            @Override
            public void onEnsureClick() { }
        });
        tipsDialog.show();
    }

    private void reportTracker(int type) {
        try {
            if (mAd != null) {
                ReportTrackerManager.getInstance().reportTrack(mAd.getTracks(type));
                mAd.trackerGroup.remove(type);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void setUp(JZDataSource jzDataSource, int screen) {
        super.setUp(jzDataSource, screen);
    }

    @Override
    public void onStateError(int what, int extra) {
        String msg = "激励视频播放失败:onStateError  what:" + what + " extra:" + extra;
        LogUtils.e("fza" + msg);
        onCompletion();
        if (mRewardVideoListener != null) {
            mRewardVideoListener.onError(ConstantPool.VideoError.PLAYINGERROR, msg);
        }
        super.onStateError(what, extra);
    }

    @Override
    public void onStatePause() {
        LogUtils.e("fza" + "onStatePause: ");
        reportTracker(ConstantPool.TrackerType.VIDEO_PAUSE_TRACKER);
        super.onStatePause();
    }

    @Override
    public void onStatePreparing() {
        LogUtils.e("fza" + "onStatePreparing: ");
        mAd = HlRewardVideoManager.getInstance().mAd;
        startCountDown();
        if (mAd != null && mAd.showAdTitle && !"gdt".equals(mAd.source)) {
            mTvAd.setText(mAd.source);
        }
        if (mAd != null && !TextUtils.isEmpty(mAd.lgsrc)) {
            ImageLoader.load(mImageAd, mAd.lgsrc);
        }
        super.onStatePreparing();
    }

    @Override
    public void onStatePlaying() {
        LogUtils.e("fza" + "onStatePlaying: ");
        super.onStatePlaying();
        JZMediaManager.instance().jzMediaInterface.setVolume(1f, 1f);
        if (mRewardVideoListener != null && !isCallOnAdShow) {
            mRewardVideoListener.onAdShow();
            isCallOnAdShow = true;
        }
        reportTracker(ConstantPool.TrackerType.DISPLAY);
        LogUtils.e("===reportTracker===DISPLAY");
        reportTracker(ConstantPool.TrackerType.VIDEO_PLAYING_0_TRACKER);
        mProgressBar.setVisibility(GONE);
    }

    private void startCountDown() {
        final String duration = mAd.reward_video.videoDuration;
        if (TextUtils.isEmpty(duration)) {
            return;
        }
        timer = new CountDownTimer(Integer.parseInt(duration)*1000, 1000) {
            public void onTick(long millisUntilFinished) {
                mTvTime.setText(String.valueOf(millisUntilFinished / 1000));
                mAd.video_progress = Integer.parseInt(duration);
                mAd.video_progress_ms = Integer.parseInt(duration)*1000;
            }

            public void onFinish() {
                mTvTime.setVisibility(GONE);
            }
        };
        timer.start();
    }

    private void onclickDetail() {
        if (mAdController != null && mAd != null && mContext != null) {
            mContext.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mAdController.clickAd(mAd);
                    if (mRewardVideoListener != null) {
                        mRewardVideoListener.onAdClick();
                        reportTracker(ConstantPool.TrackerType.VIDEO_CLICK_TRACKER);
                    }
                }
            });
        }
    }

    @Override
    public void onStateAutoComplete() {
        currentState = CURRENT_STATE_AUTO_COMPLETE;
        cancelProgressTimer();
    }

    @Override
    public void onProgress(int progress, long position, long duration) {
        if (mAd != null) {
            if (progress == 0){
                reportTracker(ConstantPool.TrackerType.VIDEO_START_TRACKER);
            } else if (!mAd.is25Report && progress > 22 && progress < 28) {
                mAd.is25Report = true;
                reportTracker(ConstantPool.TrackerType.VIDEO_PLAYING_1_TRACKER);
            } else if (!mAd.is50Report && progress > 47 && progress < 53) {
                mAd.is50Report = true;
                reportTracker(ConstantPool.TrackerType.VIDEO_PLAYING_2_TRACKER);
            } else if (!mAd.is75Report && progress > 72 && progress < 78) {
                mAd.is75Report = true;
                reportTracker(ConstantPool.TrackerType.VIDEO_PLAYING_3_TRACKER);
            } else if (progress >= 99) {
                reportTracker(ConstantPool.TrackerType.VIDEO_COMPLETE_TRACKER);
            }
        }
    }

    @Override
    public void resetProgressAndTime() {

    }

    @Override
    public void setBufferProgress(int bufferProgress) {
    }

    @Override
    public void onCompletion() {
        LogUtils.e("fza" + "onCompletion:  mVideoStateChangeListener" + (mVideoStateChangeListener == null));
        if (mVideoStateChangeListener != null) {
            mVideoStateChangeListener.videoClose();
            videoClose = true;
        }
        if (mAd != null && relative_small != null && relative_big != null && relative_center != null) {
            if (mAd.interaction_type == 2) {
                relative_small.setVisibility(GONE);
                relative_big.setVisibility(VISIBLE);
            } else {
                relative_center.setVisibility(VISIBLE);
            }
        }
        super.onCompletion();
//        onDestroy();
//        releaseTimer();
        mVideoStateChangeListener = null;
    }

    @Override
    public void onAutoCompletion() {
        LogUtils.e("fza" + "onAutoCompletion");
        reportTracker(ConstantPool.TrackerType.VIDEO_PLAYING_4_TRACKER);
        reportTracker(ConstantPool.TrackerType.VIDEO_LOADED_TRACKER);
        if (mAd != null && relative_small != null && relative_big != null && relative_center != null) {
            if (mAd.interaction_type == 2) {
                relative_small.setVisibility(GONE);
                relative_big.setVisibility(VISIBLE);
            } else {
                relative_center.setVisibility(VISIBLE);
            }
        }
//        onDestroy();
        if (mRewardVideoListener != null) {
            LogUtils.e("fza" + "onPlayEnd");
            mRewardVideoListener.onRewardArrived();
            mRewardVideoListener.onPlayEnd();
        }
        super.onAutoCompletion();
    }

    private ClickAdStateChangListener mClickAdStateChangListener = new ClickAdStateChangListener() {
        @Override
        public void onClickAdStateChang(int clickAdState) {
            switch (clickAdState) {
                case ClickAdStateChangListener.STATE_WEB_OPEN:
                    goOnPlayOnPause();
                    break;
                case ClickAdStateChangListener.STATE_WEB_CLOSE:
                    goOnPlayOnResume();
                    break;
                default:
                    break;
            }
        }
    };

    public void releaseTimer() {
        if (timer != null) {
            timer.cancel();
        }
    }

    public interface VideoStateChangeListener {
        void videoClose();
    }

    private void onDestroy() {
        mAd = null;
    }

}
