package com.halomobi.sdk;

import android.content.Context;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;

import com.halomobi.common.ConstantPool;
import com.halomobi.sdk.listener.NativeResponse;
import com.halomobi.sdk.listener.NativeUnifiedBackAdListener;
import com.halomobi.network.request.ad.bean.Ad;

import java.util.List;

public class NativeResponseImpl implements NativeResponse {
    private AdController mAdController;
    private String adTitle;
    private String adSubTitle;
    private String logoUrl;
    private int imageWidth;
    private int imageHeight;
    private double price;
    private boolean isShow = true;
    private boolean isClick;
    private List<String> mImageList;
    private Ad adInfo;

    public NativeResponseImpl(AdController adController) {
        mAdController = adController;
    }

    public void setAdTitle(String title) {
        adTitle = title;
    }

    public void setSubTitle(String subTitle) {
        adSubTitle = subTitle;
    }

    public void setLogoUrl(String logoUrlStr) {
        logoUrl = logoUrlStr;
    }

    public void setImageWidth(int width) {
        imageWidth = width;
    }

    public void setImageHeight(int height) {
        imageHeight = height;
    }

    public int getImageWidth() {
        return imageWidth;
    }

    public int getImageHeight() {
        return imageHeight;
    }

    public void setImgUrlList(List<String> imgList) {
        mImageList = imgList;
    }

    public void setPrice(double adPrice) {
        price = adPrice;
    }

    public void setAdInfo(Ad ad) {
        adInfo = ad;
    }

    @Override
    public String getTitle() {
        return adTitle;
    }

    @Override
    public List<String> getImgList() {
        return mImageList;
    }

    @Override
    public String getAdDescription() {
        return adSubTitle;
    }

    public String getLogoUrl() {
        return logoUrl;
    }


    public double getPrice() {
        return price;
    }

    public void handleClick(View v, boolean isReport) {
        mAdController.clickAd(adInfo);
        if (isReport) {
            mAdController.reportTracker(adInfo, ConstantPool.TrackerType.CLICK);
        }
    }

    @Override
    public void handleShow() {
        mAdController.reportTracker(adInfo, ConstantPool.TrackerType.DISPLAY);
        if (!TextUtils.isEmpty(adInfo.nurl)) {
            mAdController.reportTracker(adInfo, ConstantPool.TrackerType.DISPLAY_NURL);
        }
        if (!TextUtils.isEmpty(adInfo.burl)) {
            mAdController.reportTracker(adInfo, ConstantPool.TrackerType.DISPLAY_BURL);
        }
    }
    @Override
    public void registerUnifiedListener(Context context, ViewGroup container, List<View> viewList, NativeUnifiedBackAdListener listener) {
        ViewTreeObserver viewTreeObserver = container.getViewTreeObserver();
        viewTreeObserver.addOnGlobalLayoutListener(() -> {
            if (isShow) {
                isShow = false;
                handleShow();
                if (listener != null) {
                    listener.onAdShow();
                }
            }
        });
        for (View view : viewList) {
            view.setOnClickListener(v -> {
                if (!isClick) {
                    isClick = true;
                    if (listener != null) {
                        listener.onAdClick();
                    }
                }
                handleClick(v, !isClick);
            });
        }
    }
}
