package com.halomobi.sdk;

import android.app.Activity;
import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.Nullable;

import com.halomobi.download.bean.DownloadItem;
import com.halomobi.download.listener.DownloadStateListener;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.utils.KeyUtil;
import com.halomobi.utils.LogUtils;

import java.net.URISyntaxException;
import java.util.ArrayList;

/**
 * @author Feng Zhanao
 * @date 2019/3/8
 * @description
 */
public class WebViewActivity extends Activity {
    public String TAG = "WebViewActivity";
    private ArrayList<String> dUrls;
    private WebView wView;
    private TextView mTvTitle;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_webview);
        dUrls = new ArrayList<>();
        String urlStr = getIntent().getStringExtra("urlStr");
        wView = findViewById(R.id.webView);
        mTvTitle = findViewById(R.id.tv_title);
        ImageView mImgBack = findViewById(R.id.image_back);
        mImgBack.setOnClickListener(v -> {
            if (wView.canGoBack()) {
                wView.goBack();
            } else {
                finish();
            }
        });
        WebSettings webSettings = wView.getSettings();
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            webSettings.setMixedContentMode(WebSettings.MIXED_CONTENT_ALWAYS_ALLOW);
        }

        webSettings.setSupportZoom(true);
        webSettings.setJavaScriptEnabled(true);
        webSettings.setJavaScriptCanOpenWindowsAutomatically(true);
//        webSettings.setBuiltInZoomControls(true);// support zoom
//        webSettings.setDisplayZoomControls(false);
        webSettings.setUseWideViewPort(true);// 这个很关键
        webSettings.setLoadWithOverviewMode(true);
        webSettings.setBlockNetworkImage(false);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            WebView.setWebContentsDebuggingEnabled(true);
        }

        webSettings.setDomStorageEnabled(true);
        webSettings.setAllowFileAccess(true);
        WebViewClient mWebViewClient = new WebViewClient() {

            @Override
            public void onPageStarted(WebView view, String url, Bitmap favicon) {
                super.onPageStarted(view, url, favicon);
            }

            @Override
            public boolean shouldOverrideUrlLoading(WebView view, String url) {
                if (url.startsWith("http://") || url.startsWith("https://")) {
                    try {
                        WebView.HitTestResult hit = view.getHitTestResult();
                        int hitType = hit.getType();
                        if (hitType == WebView.HitTestResult.SRC_ANCHOR_TYPE) {// 点击超链接
                            LogUtils.d("shouldOverrideUrlLoading    超链     ：   " + url);
                            view.loadUrl(url);
                            return true;// 返回true浏览器执行默认的操作
                        } else if (hitType == 0) {// 重定向时hitType为0
                            LogUtils.d("shouldOverrideUrlLoading     302    ：   " + url);
                            return false;// 不捕获302重定向
                        } else {
                            return false;
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                } else {
                    openDplink(url);
                    return true;
                }
                return false;
            }
        };
        wView.setWebViewClient(mWebViewClient);
        WebChromeClient mWebChromeClient = new WebChromeClient() {
            @Override
            public void onReceivedTitle(WebView view, String title) {
                super.onReceivedTitle(view, title);
                mTvTitle.setText(title);
            }
        };

        wView.setWebChromeClient(mWebChromeClient);
        // 取消滚动条
        wView.setHorizontalScrollBarEnabled(false);//水平不显示
        wView.setVerticalScrollBarEnabled(false); //垂直不显示

        if (!TextUtils.isEmpty(urlStr)) {
            wView.loadUrl(urlStr);
        }
    }

    private DownloadItem createDownloadItem(final String url) {
        DownloadItem item = new DownloadItem();
        item.setName(KeyUtil.getMD5(url));//设置下载项目名称
        item.setDownLoadUrl(url.trim());//下载地址
        item.setListener(new DownloadStateListener() {

            @Override
            public void onDownloadError(DownloadItem item, int errorCode, String errorMsg) {
                dUrls.remove(url);
                Log.e("11111","==onDownloadError==" + errorMsg);
//                ReportErrorManager.getInstance().reportError(null, errorCode, errorMsg);
            }

            @Override
            public void onDownloadStart(DownloadItem item, Ad adInfo) {
            }

            @Override
            public void onDownloaded(DownloadItem item, Ad adInfo) {
                dUrls.remove(url);
            }

            @Override
            public void onInstalled(DownloadItem item, Ad adInfo) {
            }

            @Override
            public void onOpened(DownloadItem item, Ad adInfo) {
            }

        });

        return item;
    }

    private void openDplink(String deepLink) {
        try {
            Intent deepLinkIntent = Intent.parseUri(deepLink, Intent.URI_INTENT_SCHEME);
            deepLinkIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            ResolveInfo resolveInfo = getPackageManager().resolveActivity(deepLinkIntent, PackageManager.MATCH_DEFAULT_ONLY);
            if (resolveInfo != null) {
                startActivity(deepLinkIntent);
            }
        } catch (ActivityNotFoundException e) {
            Log.w("DeepLink NotFound  :  ", deepLink);
        } catch (URISyntaxException e) {
            e.printStackTrace();
            Log.w("DeepLink NotFound  :  ", deepLink);
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if (wView != null) {
                if (wView.canGoBack()) {
                    wView.goBack();
                    return true;
                } else {
                    finish();
                }
            }
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (wView != null) {
            wView.destroy();
        }
    }
}
